"""Product block for :class:`Opengear` products."""

import ipaddress

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.site import (
    SiteBlock,
    SiteBlockInactive,
    SiteBlockProvisioning,
)


class OpengearBlockInactive(
    ProductBlockModel,
    lifecycle=[SubscriptionLifecycle.INITIAL],
    product_block_name="OpengearBlock",
):
    """An Opengear that's being currently inactive. See :class:`OpengearBlock`."""

    opengear_hostname: str | None = None
    opengear_site: SiteBlockInactive | None = None
    opengear_wan_address: ipaddress.IPv4Address | None = None
    opengear_wan_netmask: ipaddress.IPv4Address | None = None
    opengear_wan_gateway: ipaddress.IPv4Address | None = None


class OpengearBlockProvisioning(OpengearBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """An Opengear that's being provisioned. See :class:`OpengearBlock`."""

    opengear_hostname: str
    opengear_site: SiteBlockProvisioning
    opengear_wan_address: ipaddress.IPv4Address | None = None
    opengear_wan_netmask: ipaddress.IPv4Address | None = None
    opengear_wan_gateway: ipaddress.IPv4Address | None = None


class OpengearBlock(OpengearBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """An Opengear that's currently deployed in the network."""

    #: The hostname of the Opengear device.
    opengear_hostname: str
    #: The site where the Opengear device is located.
    opengear_site: SiteBlock
    #: The :term:`WAN` address of the Opengear device.
    opengear_wan_address: ipaddress.IPv4Address
    #: The :term:`WAN` netmask of the Opengear device.
    opengear_wan_netmask: ipaddress.IPv4Address
    #: The :term:`WAN` gateway of the Opengear device.
    opengear_wan_gateway: ipaddress.IPv4Address
